﻿using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Regression {
  [StorableType("4CFC2616-9B32-40EE-BAE7-25E8BFBAF5BC")]
  public abstract class
    SymbolicRegressionMultiObjectiveSplittingEvaluator : SymbolicRegressionMultiObjectiveEvaluator {
    private const string MinSplittingWidthParameterName    = "Minimum splitting width";
    private const string MaxSplittingDepthParameterName    = "Maximum splitting depth";
    private const string UseIntervalSplittingParameterName = "Use interval splitting";

    public IFixedValueParameter<IntValue> MinSplittingWidthParameter =>
      (IFixedValueParameter<IntValue>) Parameters[MinSplittingWidthParameterName];

    public IFixedValueParameter<IntValue> MaxSplittingDepthParameter =>
      (IFixedValueParameter<IntValue>) Parameters[MaxSplittingDepthParameterName];

    public IFixedValueParameter<BoolValue> UseIntervalSplittingParameter =>
      (IFixedValueParameter<BoolValue>) Parameters[UseIntervalSplittingParameterName];

    public int MinSplittingWidth {
      get => MinSplittingWidthParameter.Value.Value;
      set => MinSplittingWidthParameter.Value.Value = value;
    }

    public int MaxSplittingDepth {
      get => MaxSplittingDepthParameter.Value.Value;
      set => MaxSplittingDepthParameter.Value.Value = value;
    }

    public bool UseIntervalSplitting {
      get => UseIntervalSplittingParameter.Value.Value;
      set => UseIntervalSplittingParameter.Value.Value = value;
    }

    [StorableConstructor]
    protected SymbolicRegressionMultiObjectiveSplittingEvaluator(StorableConstructorFlag _) : base(_) { }

    protected SymbolicRegressionMultiObjectiveSplittingEvaluator(
      SymbolicRegressionMultiObjectiveSplittingEvaluator original, Cloner cloner) : base(original, cloner) { }

    protected SymbolicRegressionMultiObjectiveSplittingEvaluator() {
      Parameters.Add(new FixedValueParameter<IntValue>(MinSplittingWidthParameterName,
        "The minimum interval width before the splitting ends.", new IntValue(0)));
      Parameters.Add(new FixedValueParameter<IntValue>(MaxSplittingDepthParameterName,
        "The maximum recursion depth before the splitting ends.", new IntValue(5)));
      Parameters.Add(new FixedValueParameter<BoolValue>(UseIntervalSplittingParameterName, "", new BoolValue(false)));
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      if (!Parameters.ContainsKey(MinSplittingWidthParameterName)) {
        Parameters.Add(new FixedValueParameter<IntValue>(MinSplittingWidthParameterName,
          "The minimum interval width before the splitting ends.", new IntValue(0)));
      }

      if (!Parameters.ContainsKey(MaxSplittingDepthParameterName)) {
        Parameters.Add(new FixedValueParameter<IntValue>(MaxSplittingDepthParameterName,
          "The maximum recursion depth before the splitting ends.", new IntValue(5)));
      }

      if (!Parameters.ContainsKey(UseIntervalSplittingParameterName)) {
        Parameters.Add(new FixedValueParameter<BoolValue>(UseIntervalSplittingParameterName, "", new BoolValue(false)));
      }
    }
  }
}